(* Parameter access *)
FUNCTION_BLOCK FB_SikoAp20Prm
	////////////////////////////////////////////////////////////////////////////////////
	// Name: FB_SikoAp20Prm
	////////////////////////////////////////////////////////////////////////////////////
	// Version history
	////////////////////////////////////////////////////////////////////////////////////
	// Date        | Vers. 		| created               | Author          | Remark
	// dd.mm.yyyy  |       		| with                  |                 | 
	// ---------------------------------------------------------------------------------
	// 21.05.2019  | 1.00.00	| AS V4.3.3.196		    | SIKO GmbH / ch  | first release
	////////////////////////////////////////////////////////////////////////////////////

	(* Init *)
	IF ((nIndex = 0) AND NOT bcExecute) THEN
		bsDone := 0;
		bsBusy := 0;
		bsError := 0;
		nStatus := 0;
		nErrorInfo := 0;
		nReceiveValue := 0;
		vbOsrExecute := 0;
		vnSdoReadStep := 0;
		vnSdoWriteStep := 0;
		vbWriteCommand := 0;
		vnDataLength := 0;
		vnRxBuffer := 0;
		vnTxBuffer := 0;
		nReceiveValue := 0;
	END_IF;
	
	(* Edge Detection *)
	fbR_TRIG1(CLK:= bcExecute);
	vbOsrExecute := fbR_TRIG1.Q;
	
	(* new command on rising edge *)
	IF vbOsrExecute THEN
		bsDone := 0;
		bsBusy := 1;
		bsError := 0; 
		nStatus := 0;
		nErrorInfo := 0;
		IF(bcReadWrite)THEN
			// write command
			vbWriteCommand := 1;
		ELSE
			// read command
			vbWriteCommand := 0;
		END_IF;
	
		//always start with read request
		vnSdoReadStep := 1;
	END_IF;

	(* handle read request *)

	// init read request
	IF (vnSdoReadStep = 1) THEN
		//--> common settings for read requests
		fbEplSDORead.index := nIndex;
		fbEplSDORead.subindex := 0;
		fbEplSDORead.pData := ADR(vnRxBuffer);
		fbEplSDORead.datalen := SIZEOF(vnRxBuffer);
		//<-- 
	
		// clear input buffer
		vnRxBuffer := 0;
		nReceiveValue := 0;

		// execute read request
		vnSdoReadStep := 2;
	END_IF;

	// execute read request
	IF vnSdoReadStep = 2 THEN
		fbEplSDORead(enable := TRUE, node:=DC_nNodeId, pDevice:= ADR(DC_sDevicename));
		IF fbEplSDORead.status <> 16#FFFF THEN
			IF fbEplSDORead.status = 0 THEN
				(* read success *)
				vnDataLength := fbEplSDORead.readlen;
				IF (vbWriteCommand) THEN
					vnSdoWriteStep := 1;
				ELSE
					IF (vnDataLength = 1) THEN
						nReceiveValue := vnRxBuffer AND 16#000000FF;
					ELSIF (vnDataLength = 2) THEN
						(* Integer16 representation of Index 0x2008 AND 0x202D *)
						IF (vnRxBuffer.15) AND (nIndex = 16#2008 OR nIndex = 16#202D) THEN
							nReceiveValue := vnRxBuffer;
							nReceiveValue := nReceiveValue + DWORD_TO_DINT(16#FFFF0000);
						ELSE
							nReceiveValue := vnRxBuffer AND 16#0000FFFF; 	
						END_IF;
					ELSE
						nReceiveValue := vnRxBuffer;		
					END_IF;
					
					// set status bits
					bsDone := 1;
					bsBusy := 0;
					bsError := 0;
				END_IF;
			ELSE
				(* read bsError *)
				bsDone:=FALSE;
				bsBusy:=FALSE;
				bsError:=TRUE;
			END_IF;
			nStatus := fbEplSDORead.status;
			nErrorInfo := fbEplSDORead.errorinfo;
			fbEplSDORead.enable := FALSE;
			vnSdoReadStep := 0;
		END_IF;
	END_IF;

	(* handle write request *)
	
	// init write request
	IF (vnSdoWriteStep = 1) THEN
		//--> prepare write request
		fbEplSDOWrite.index := nIndex;
		fbEplSDOWrite.subindex := 0;
		fbEplSDOWrite.pData := ADR(vnTxBuffer);
		fbEplSDOWrite.datalen := vnDataLength;
		vnTxBuffer := nWriteValue;
		vnSdoWriteStep := 2;
	END_IF;

	// send message
	IF (vnSdoWriteStep = 2) THEN
		fbEplSDOWrite(enable:=TRUE, node:= DC_nNodeId, pDevice:= ADR(DC_sDevicename));
		IF (fbEplSDOWrite.status <> 16#FFFF) THEN
			IF fbEplSDOWrite.status = 0 THEN
				nReceiveValue := nWriteValue;
				bsDone:=TRUE;
				bsBusy:=FALSE;
				bsError:=FALSE;
			ELSE
				(* write bsError *)
				bsDone:=FALSE;
				bsBusy:=FALSE;
				bsError:=TRUE;
			END_IF;
			nStatus := fbEplSDOWrite.status;
			nErrorInfo := fbEplSDOWrite.errorinfo;
			fbEplSDOWrite.enable := FALSE;
			vnSdoWriteStep := 0;
		END_IF;
	END_IF;

END_FUNCTION_BLOCK